% SVS/MVS SIGNAL GENERATOR
%
% Creating and plotting stochastic vestibular stimulation (SVS) signals
% and multisine vestibular stimulation signals (MVS).
%
% Signal generation tool as presented in Forbes et al. 2013, PLOS ONE
%
% version 0.1: first-release, November 10,2013
% Copyright (C) 2013, Patrick A. Forbes
% This software and signals may be used, copied, or redistributed as long as
% it is not sold and this copyright notice is reproduced on each copy made.
% These signals and software are provided as is, without any express or 
% implied warranties whatsoever. 
%
% Please acknowledge the use of these signals and software in any publication
% by refering to the article: 
%  "Electrical vestibular stimuli to enhance vestibulo-motor output and
%  improve subject comfort."
%  Forbes, Dakin, Geers, Vlaar, Happee, Siegmund, Schouten, Blouin,
%  PLOS ONE, 2013, accepted.
%
% Patrick A. Forbes
% Delft Laboratory for NeuroMuscular Control
% Department of BioMechanical Engineering
% Delft University of Technology
%
% Mekelweg 2
% 2628 CD Delft
% The Netherlands
% e-mail: p.a.forbes@tudelft.nl
% url:    www.3me.tudelft.nl/nmc

% version November 10, 2013
clear all;
close all;

% Preliminary information
Fs    = 2000;           % Sampling rate [Hz]
T     = 90;             % Signal duration [s]
dt    = 1/Fs;           % Time step [s]
N     = T*Fs;           % Number of samples
Ts    = 2.5;            % Segment duration [s]
Ns    = Ts*Fs;          % Number of samples in segment duration
fcut  = 25;             % Bandwidth of signal being created [Hz]
df    = 1/Ts;           % Frequency resolution [Hz]

t     = (0:N-1)'.*dt;   % Time vector
f     = (1:N)'./T;      % Frequency vector
ts    = (0:Ns-1)'.*dt;  % Segment time vector
fs    = (1:Ns)'./Ts;    % Segment frequency vector
fp    = (df:df:fcut)';  % Frequency vector up to fcut - all points, frequency resolution of 1/Ts
npt   = length(fp);     % Number of points in fcut frequency vector at frequency resolution of 1/Ts

meth  = 2;              % Method for generating signals

% METHOD 1: optimized cresting - uses msinclip and msinprep functions
% Signals described in manuscript (Forbes et al. 2013) used this method
% Functions can be found in Frequency Domain System Identification Toolbox (FDIDENT)
% Details can be found at http://home.mit.bme.hu/~kollar/fdident/
% Kollar, Istvan, "Frequency Domain System Identificaton Toolbox for Matlab", Gamax Ltd, Budapest, 2004-2009.

% METHOD 2 : generates 1000 multisines, chooses signal with lowest crest
% factor, avoids the need for msinclip and msinprep
% Note: better crest factors can be achieved using optimization in METHOD 1
                        

%% SVS signal
% Setting up parameters 
[b, a]   = butter(6, fcut/(Fs/2),'low'); % 6th order butterworth filter
maxAmp   = 5;    % Maximum signal peak

% Generating 100 signals, and select 'best' one
svs      = filtfilt(b, a, randn(N+Fs,100));    % include one extra second
svs      = svs(Fs/2+1:end-Fs/2,:);              % remove extra second to avoid transient effects of filter
svs      = maxAmp * svs ./ (ones(N,1)*max(abs(svs),[],1));
% select 'best' one, i.e. the lowest crest factor (to remove extreme outliers)
crest    = max(abs(svs),[],1)./std(svs,[],1);
[~,cr_i] = min(crest);
svs      = svs(:,cr_i);

% Preparing autospectrum for plotting
svstmp   = reshape(svs, Ts*Fs, T/Ts);
A        = fft(svstmp);    
SVS      = 1/N.*real(conj(A).*A);  
mSVS     = mean(SVS,2);

%% MVS-S signal
% Defining frequency vector
fp               = roundn((df:df:fcut)',-1);
np               = length(fp);

% Generate signal
if meth == 1      % METHOD 1 - generate signal: optimized cresting - uses msinclip and msinprep functions
    Bcrs             = msinclip(fp, exp(2*pi*1j*rand(size(fp))), [], 'nograph');
    [mvss,~]         = msinprep(fp, Bcrs, N, Fs, 'screen');
elseif meth == 2  % METHOD 2 - generate signal: generates 1000 multisines, chooses one with lowest crest factor
    nms              = 1000;                  % number of multisines
    indB             = find(ismember(fs, fp)); % indices of selected frequencies
    Brnd             = zeros(Ns,nms);          % full frequency vector
    Brnd(indB+1,:)   = 1.*exp(2*pi*1j*rand(length(fp),nms));
    mvss             = 2*real(ifft(Brnd));
    crest            = max(abs(mvss),[],1)./std(mvss,[],1);
    [~,cr_i]         = min(crest);
    mvss             = mvss(:,cr_i);
end

mvsstmp          = repmat(mvss,[1 N/Ns]);
mvss             = mvsstmp(:);

% Preparing autospectrum for scaling - only temporarily used
B                = fft(mvsstmp);    
MVSSt            = 1/N.*real(conj(B).*B);
mMVSSt           = mean(MVSSt,2);

% Scale the signal to have same power in spectral domain
scale            = sqrt(max(mMVSSt)/mean(mSVS(1:npt)));
mvss             = mvss/scale;

% Preparing autospectrum for plotting
mvsstmp          = reshape(mvss, Ts*Fs, T/Ts);
B                = fft(mvsstmp);    
MVSS             = 1/N.*real(conj(B).*B);  
mMVSS            = mean(MVSS,2);


%% MVS-S signal
% Defining frequency vector
fp               = roundn((df:4*df:fcut)',-1);

% Generate signal
if meth == 1      % METHOD 1 - generate signal: optimized cresting - uses msinclip and msinprep functions
    Ccrs             = msinclip(fp, exp(2*pi*1j*rand(size(fp))), [], 'nograph');
    [mvs4, dum]      = msinprep(fp, Ccrs, N, Fs, 'screen');
elseif meth == 2  % METHOD 2 - generate signal: generates 1000 multisines, chooses one with lowest crest factor
    nms              = 1000;                  % number of multisines
    indC             = find(ismember(fs, fp)); % indices of selected frequencies
    Crnd             = zeros(Ns,nms);          % full frequency vector
    Crnd(indC+1,:)   = 1.*exp(2*pi*1j*rand(length(fp),nms));
    mvs4             = 2*real(ifft(Crnd));
    crest            = max(abs(mvs4),[],1)./std(mvs4,[],1);
    [~,cr_i]         = min(crest);
    mvs4             = mvs4(:,cr_i);
end

mvs4tmp          = repmat(mvs4,[1 N/Ns]);
mvs4             = mvs4tmp(:);

% Preparing autospectrum for scaling - only temporarily used
mvs4tmp          = reshape(mvs4, Ts*Fs, T/Ts);
C                = fft(mvs4tmp);    
MVS4t            = 1/N.*real(conj(C).*C);  
mMVS4t           = mean(MVS4t,2);

% Scale the signal to have same power in spectral domain
scale            = sqrt(max(mMVS4t)/mean(mSVS(1:npt)));
mvs4             = mvs4/scale;

% Preparing autospectrum for plotting
mvs4tmp          = reshape(mvs4, Ts*Fs, T/Ts);
C                = fft(mvs4tmp);    
MVS4             = 1/N.*real(conj(C).*C);  
mMVS4            = mean(MVS4,2);

%% MVS-L signal
% Setting up frequency vector
np_log           = 80;                                       % Number of points in log distribution (0-100 Hz)
fp               = logspace(log10(df),log10(100),np_log);    % Create log distribution up to 100 Hz
fp_ind           = round(fp/df);                             % Round to frequency resolution (df = 1/Ts Hz)
fp_ind           = unique(fp_ind);                           % Use only unique indices
fp               = fs(fp_ind);                               % Use only unique frequency points

find_up          = find(fp>fcut,1,'first');                  % Limit frequency points to those under fcut Hz
fp_ind           = fp_ind(1:find_up-1);                      %
fp               = fs(fp_ind);                               %

% Generate signal
if meth == 1      % METHOD 1 - generate signal: optimized cresting - uses msinclip and msinprep functions
    Dcrs               = msinclip(fp, exp(2*pi*1j*rand(size(fp))), [], 'nograph');
    [mvsl, dum]        = msinprep(fp, Dcrs, N, Fs, 'screen');
elseif meth == 2  % METHOD 2 - generate signal: generates 1000 multisines, chooses one with lowest crest factor
    nms              = 1000;                  % number of multisines
    indD             = find(ismember(fs, fp)); % indices of selected frequencies
    Drnd             = zeros(Ns,nms);          % full frequency vector
    Drnd(indD+1,:)   = 1.*exp(2*pi*1j*rand(length(fp),nms));
    mvsl             = 2*real(ifft(Drnd));
    crest            = max(abs(mvsl),[],1)./std(mvsl,[],1);
    [~,cr_i]         = min(crest);
    mvsl             = mvsl(:,cr_i);
end

mvsltmp          = repmat(mvsl,[1 N/Ns]);
mvsl             = mvsltmp(:);

% Preparing autospectrum for scaling - only temporarily used
mvsltmp          = reshape(mvsl, Ts*Fs, T/Ts);
D                = fft(mvsltmp);    
MVSLt            = 1/N.*real(conj(D).*D);  
mMVSLt           = mean(MVSLt,2);

% Scale the signal to have same power in spectral domain
scale            = sqrt(max(mMVSLt)/mean(mSVS(1:npt)));
mvsl             = mvsl/scale;

% Preparing autospectrum for plotting
mvsltmp          = reshape(mvsl, Ts*Fs, T/Ts);
D                = fft(mvsltmp);    
MVSL             = 1/N.*real(conj(D).*D);  
mMVSL            = mean(MVSL,2);

%% Plotting the generated signals
% Finding the indices for multisine signals for plotting
iMVSS = find(mMVSS(1:end/2) > mean(mMVSS(1:end/2)));
iMVS4 = find(mMVS4(1:end/2) > mean(mMVS4(1:end/2)));
iMVSL = find(mMVSL(1:end/2) > mean(mMVSL(1:end/2)));

% Plotting signals 
seg = find(max(abs(svstmp)) == 5);    % Segment number in SVS where amplitude is at 5 mA (can be 1-36)
fig = figure(1);
set(fig,'Position',[100 100 1300 600],'Color',[1 1 1],'Name','Signal Plots');
ylit = 5; ylif = 4;  % y-axis limits in time and frequency domains

% Plotting in time domain
subplot(2,4,1);
plot(ts, svstmp(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
ylabel('Current [mA]','FontSize',16); xlabel('Time [s]','FontSize',16);
title('SVS Signal','FontSize',16);
subplot(2,4,2);
plot(ts, mvsstmp(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-S Signal','FontSize',16);
subplot(2,4,3);
plot(ts, mvs4tmp(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-4 Signal','FontSize',16);
subplot(2,4,4);
plot(ts, mvsltmp(:,seg));
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-L Signal','FontSize',16);

% Plotting in frequency domain
subplot(2,4,5);
plot(fs,mSVS);
xlim([0 27]); ylim([0 ylif]); box off
ylabel('Autospectrum [-]','FontSize',16); xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,6)
stem(fs(iMVSS),mMVSS(iMVSS),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,7)
stem(fs(iMVS4),mMVS4(iMVS4),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,8)
stem(fs(iMVSL),mMVSL(iMVSL),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);

%% Saving signals

filename = ['P1_SVS_' num2str(T) 's_' num2str(roundn(Fs/1000,-1)) 'kHz.txt'];
save(filename,'svs','-ascii');
filename = ['P1_MVS_S_' num2str(T) 's_' num2str(roundn(Fs/1000,-1)) 'kHz.txt'];
save(filename,'mvss','-ascii');
filename = ['P1_MVS_4_' num2str(T) 's_' num2str(roundn(Fs/1000,-1)) 'kHz.txt'];
save(filename,'mvs4','-ascii');
filename = ['P1_MVS_L_' num2str(T) 's_' num2str(roundn(Fs/1000,-1)) 'kHz.txt'];
save(filename,'mvsl','-ascii');