% SVS/MVS signal loading and plotting
%
% Loading and plotting of stochastic vestibular stimulation (SVS) signals
% and multisine vestibular stimulation signals (MVS).
%
% Signals presented in Forbes et al. 2013, PLOS ONE
%
% version 0.1: first-release, November 10,2013
% Copyright (C) 2013, Patrick A. Forbes
% This software and signals may be used, copied, or redistributed as long as
% it is not sold and this copyright notice is reproduced on each copy made.
% These signals and software are provided as is, without any express or 
% implied warranties whatsoever. 
%
% Please acknowledge the use of this these signals and software in any publication
% by refering to the article: 
%  "Electrical vestibular stimuli to enhance vestibulo-motor output and
%  improve subject comfort."
%  Forbes, Dakin, Geers, Vlaar, Happee, Siegmund, Schouten, Blouin,
%  PLOS ONE, 2013, accepted.
%
% Patrick A. Forbes
% Delft Laboratory for NeuroMuscular Control
% Department of BioMechanical Engineering
% Delft University of Technology
%
% Mekelweg 2
% 2628 CD Delft
% The Netherlands
% e-mail: p.a.forbes@tudelft.nl
% url:    www.3me.tudelft.nl/nmc

% version November 10, 2013
clear all;
close all;

% Preliminary info for plotting
Fs = 2000;           % Sampling rate [Hz]
T  = 90;             % Signal duration [s]
dt = 1/Fs;           % Time step [s]
N  = T*Fs;           % Number of samples
Ts = 2.5;            % Segment duration [s]
Ns = Ts*Fs;          % Number of samples in segment duration

t  = (0:N-1)'.*dt;   % Time vector
f  = (1:N)'./T;      % Frequency vector
ts = (0:Ns-1)'.*dt;  % Segment time vector
fs = (1:Ns)'./Ts;    % Segment frequency vector

% Loading signals, note .txt files should be in same folder as .m script
% Reshaping into 2.5 s (Ts) segments for plotting and spectral analysis
svs  = load('SVS_90s_2kHz.txt');        svs  = reshape(svs,[Ns N/Ns]);
mvss = load('MVS_S_90s_2kHz.txt');      mvss = reshape(mvss,[Ns N/Ns]);
mvs4 = load('MVS_4_90s_2kHz.txt');      mvs4 = reshape(mvs4,[Ns N/Ns]);
mvsl = load('MVS_L_90s_2kHz.txt');      mvsl = reshape(mvsl,[Ns N/Ns]);

% Calculating autospectrum of each signal
TEMP = fft(svs);    SVS  = 1/N.*real(conj(TEMP).*TEMP);    mSVS  = mean(SVS,2);
TEMP = fft(mvss);   MVSS = 1/N.*real(conj(TEMP).*TEMP);    mMVSS = mean(MVSS,2);
TEMP = fft(mvs4);   MVS4 = 1/N.*real(conj(TEMP).*TEMP);    mMVS4 = mean(MVS4,2);
TEMP = fft(mvsl);   MVSL = 1/N.*real(conj(TEMP).*TEMP);    mMVSL = mean(MVSL,2);
clear TEMP;

% Finding the indices for multisine signals for plotting
iMVSS = find(mMVSS(1:end/2) > mean(mMVSS(1:end/2)));
iMVS4 = find(mMVS4(1:end/2) > mean(mMVS4(1:end/2)));
iMVSL = find(mMVSL(1:end/2) > mean(mMVSL(1:end/2)));

% Plotting signals 
seg = 3;            % Select which segment to plot (can be 1-36)
fig = figure(1);
set(fig,'Position',[100 100 1300 600],'Color',[1 1 1],'Name','Signal Plots');
ylit = 5; ylif = 2.5;  % y-axis limits in time and frequency domains

% Plotting in time domain
subplot(2,4,1);
plot(ts, svs(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
ylabel('Current [mA]','FontSize',16); xlabel('Time [s]','FontSize',16);
title('SVS Signal','FontSize',16);
subplot(2,4,2);
plot(ts, mvss(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-S Signal','FontSize',16);
subplot(2,4,3);
plot(ts, mvs4(:,seg)); 
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-4 Signal','FontSize',16);
subplot(2,4,4);
plot(ts, mvsl(:,seg));
xlim([0 2.5]); ylim([-ylit ylit]); box off
xlabel('Time [s]','FontSize',16);  title('MVS-L Signal','FontSize',16);

% Plotting in frequency domain
subplot(2,4,5);
plot(fs,mSVS);
xlim([0 27]); ylim([0 ylif]); box off
ylabel('Autospectrum [-]','FontSize',16); xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,6)
stem(fs(iMVSS),mMVSS(iMVSS),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,7)
stem(fs(iMVS4),mMVS4(iMVS4),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);
subplot(2,4,8)
stem(fs(iMVSL),mMVSL(iMVSL),'Filled','MarkerSize',4,'MarkerEdgeColor','r','MarkerFaceColor','r'); 
xlim([0 27]); ylim([0 ylif]); box off
xlabel('Frequency [Hz]','FontSize',16);

